#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "Settings.h"
#include "About.h"
#include "sg633usb.h"

#pragma package(smart_init)
#pragma link "FloatEdit"
#pragma resource "*.dfm"
TMainForm *MainForm;

const double AMP2RMS = 0.70710678118654752440084436210485;
const double RMS2AMP = 1.4142135623730950488016887242097;

//---------------------------------------------------------------------------

__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
}

//------------------------- Read INI file: ----------------------------------

void __fastcall TMainForm::ReadConfig(void)
{
  TMemIniFile *ini; AnsiString Section;
  ini = new TMemIniFile(ChangeFileExt(Application->ExeName,".ini" ));
  Section = "General";
  Left   = ini->ReadInteger(Section,"Left",Left);
  Top    = ini->ReadInteger(Section,"Top",Top);
  Section = "Mode";
  Lock = ini->ReadBool(Section,"Lock",0);
  Mute = ini->ReadBool(Section,"Mute",0);
  mnReview->Checked = ini->ReadBool(Section,"Review",0);
  Autorecall->Checked = ini->ReadBool(Section,"Autorecall",1);
  Autoreload->Checked = !Autorecall->Checked;
  mnRms->Checked = ini->ReadBool(Section,"RMS",1);
  delete ini;

  ReadData(ChangeFileExt(Application->ExeName,".ini" ));
}

//------------------------- Write INI file: ---------------------------------

void __fastcall TMainForm::SaveConfig(void)
{
  TMemIniFile *ini; AnsiString Section;
  ini = new TMemIniFile(ChangeFileExt(Application->ExeName,".ini" ));
  Section = "General";
  ini->WriteInteger(Section,"Left",Left);
  ini->WriteInteger(Section,"Top",Top);
  Section = "Mode";
  ini->WriteBool(Section,"Lock",Lock);
  ini->WriteBool(Section,"Mute",Mute);
  ini->WriteBool(Section,"Review",mnReview->Checked);
  ini->WriteBool(Section,"Autorecall",Autorecall->Checked);
  ini->WriteBool(Section,"RMS",mnRms->Checked);

  try { ini->UpdateFile(); } catch (Exception &E)
   { MessageDlg(E.Message, mtError, TMsgDlgButtons() << mbOK, 0); }
  delete ini;

  SaveData(ChangeFileExt(Application->ExeName,".ini" ));
}

//------------------------ Read data file: ----------------------------------

void __fastcall TMainForm::ReadData(AnsiString FileName)
{
  TMemIniFile *ini; AnsiString Section;
  ini = new TMemIniFile(FileName);

  Section = "OUT A";
  ValOuts[OUT_A][PAR_M] = ini->ReadInteger(Section,"M",0);
  ValOuts[OUT_A][PAR_F] = ini->ReadFloat(Section,  "F",1);
  ValOuts[OUT_A][PAR_P] = ini->ReadFloat(Section,  "P",0);
  ValOuts[OUT_A][PAR_A] = ini->ReadFloat(Section,  "A",1);

  Section = "OUT B";
  ValOuts[OUT_B][PAR_M] = ini->ReadInteger(Section,"M",0);
  ValOuts[OUT_B][PAR_F] = ini->ReadFloat(Section,  "F",1);
  ValOuts[OUT_B][PAR_P] = ini->ReadFloat(Section,  "P",0);
  ValOuts[OUT_B][PAR_A] = ini->ReadFloat(Section,  "A",1);

  Section = "OUT C";
  ValOuts[OUT_C][PAR_M] = ini->ReadInteger(Section,"M",0);
  ValOuts[OUT_C][PAR_F] = ini->ReadFloat(Section,  "F",1);
  ValOuts[OUT_C][PAR_P] = ini->ReadFloat(Section,  "P",0);
  ValOuts[OUT_C][PAR_A] = ini->ReadFloat(Section,  "A",1);

  delete ini;
  ShowAll();
}

//------------------------ Write data file: ---------------------------------

void __fastcall TMainForm::SaveData(AnsiString FileName)
{
  TMemIniFile *ini; AnsiString Section;
  ini = new TMemIniFile(FileName);

  Section = "OUT A";
  ini->WriteInteger(Section,"M",ValOuts[OUT_A][PAR_M]);
  ini->WriteFloat(Section,  "F",ValOuts[OUT_A][PAR_F]);
  ini->WriteFloat(Section,  "P",ValOuts[OUT_A][PAR_P]);
  ini->WriteFloat(Section,  "A",ValOuts[OUT_A][PAR_A]);

  Section = "OUT B";
  ini->WriteInteger(Section,"M",ValOuts[OUT_B][PAR_M]);
  ini->WriteFloat(Section,  "F",ValOuts[OUT_B][PAR_F]);
  ini->WriteFloat(Section,  "P",ValOuts[OUT_B][PAR_P]);
  ini->WriteFloat(Section,  "A",ValOuts[OUT_B][PAR_A]);

  Section = "OUT C";
  ini->WriteInteger(Section,"M",ValOuts[OUT_C][PAR_M]);
  ini->WriteFloat(Section,  "F",ValOuts[OUT_C][PAR_F]);
  ini->WriteFloat(Section,  "P",ValOuts[OUT_C][PAR_P]);
  ini->WriteFloat(Section,  "A",ValOuts[OUT_C][PAR_A]);

  try { ini->UpdateFile(); } catch (Exception &E)
   { MessageDlg(E.Message, mtError, TMsgDlgButtons() << mbOK, 0); }
  delete ini;
}

//--------------------------- Recall values: --------------------------------

void __fastcall TMainForm::Recall(void)
{
  for(int j = 0; j < OUTS; j++)
    for(int i = 0; i < POUTS_MAX; i++)
      Cmd_GetPar(j, i, ValOuts[j][i]);
  int n; Cmd_GetMode(n);
  Lock = n;
  ShowAll();
}

//--------------------------- Reload values: --------------------------------

void __fastcall TMainForm::Reload(void)
{
  int ch, par, val;

  for(int j = 0; j < OUTS; j++)
    for(int i = 0; i < POUTS_MAX; i++)
      Cmd_SetPar(j, i, ValOuts[j][i]);
  Cmd_SetPar(OUT_A, DRAW | PAR_P, ValOuts[OUT_A][PAR_P]);
}

//--------------------- Convert value to parameter: -------------------------

int __fastcall TMainForm::Val2Par(double v)
{
  return((int)(v + ((v < 0)? - 0.5 : 0.5)));
}

//------------------------- Show all parameters: ----------------------------

void __fastcall TMainForm::ShowAll(void)
{
  FLock = 1;
  //channel A:
  cbMa->ItemIndex = ValOuts[OUT_A][PAR_M];
  feFa->Value = ValOuts[OUT_A][PAR_F] / 1E3;
  fePa->Value = ValOuts[OUT_A][PAR_P] / 1E1;
  double va = ValOuts[OUT_A][PAR_A] / 1E2; //V
  if(mnRms->Checked)
  {
    feAa->Value = va * AMP2RMS;
    feAa->MaxValue = 10 * AMP2RMS;
  }
  else
  {
    feAa->MaxValue = 10;
    feAa->Value = va;
  }
  //channel B:
  cbMb->ItemIndex = ValOuts[OUT_A][PAR_M]; //show OUT_A mode
  if(ValOuts[OUT_A][PAR_M] == MD_COMBO)
    feFb->Value = ValOuts[OUT_A][PAR_F] / 1E3;
      else feFb->Value = ValOuts[OUT_B][PAR_F] / 1E3;
  fePb->Value = ValOuts[OUT_B][PAR_P] / 1E1;
  double vb = ValOuts[OUT_B][PAR_A] / 1E2; //V
  if(mnRms->Checked)
  {
    feAb->Value = vb * AMP2RMS;
    feAb->MaxValue = 10 * AMP2RMS;
  }
  else
  {
    feAb->MaxValue = 10;
    feAb->Value = vb;
  }
  //channel C:
  cbMc->ItemIndex = ValOuts[OUT_A][PAR_M]; //show OUT_A mode
  if(ValOuts[OUT_A][PAR_M] == MD_COMBO)
    feFc->Value = ValOuts[OUT_A][PAR_F] / 1E3;
      else feFc->Value = ValOuts[OUT_C][PAR_F] / 1E3;
  fePc->Value = ValOuts[OUT_C][PAR_P] / 1E1;
  double vc = ValOuts[OUT_C][PAR_A] / 1E2; //V
  if(mnRms->Checked)
  {
    feAc->Value = vc * AMP2RMS;
    feAc->MaxValue = 10 * AMP2RMS;
  }
  else
  {
    feAc->MaxValue = 10;
    feAc->Value = vc;
  }
  //Lock, Mute:
  mnLock->Checked = Lock;
  mnMute->Checked = Mute;
  FLock = 0;
}

//---------------------------- Form Create: ---------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  FLock = 1;
  ReadConfig();
  GroupBox1->Enabled = !mnReview->Checked;
  GroupBox2->Enabled = !mnReview->Checked;
  GroupBox3->Enabled = !mnReview->Checked;
  lbLevelA->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  lbLevelB->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  lbLevelC->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  FLock = 0;
  FConnected = 0;
  StatusBar->SimpleText = "OFF-Line";
  Timer1->Enabled = 1;
}

//---------------------------- Form Close: ----------------------------------

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  try
  {
    Cmd_SetMode(0); //unlock local controls
  }
  catch (const EHardwareException &e) { }
  SaveConfig();
  if(FConnected) SG633_CloseDevice();
}

//------------------------------ Open: --------------------------------------

void __fastcall TMainForm::mnOpenClick(TObject *Sender)
{
  OpenDialog->Title = "Open file";
  OpenDialog->InitialDir = ExtractFilePath(ParamStr(0));
  OpenDialog->Filter = "Params files (*.par)|*.par|";
  OpenDialog->DefaultExt = "par";
  if(OpenDialog->Execute())
  {
    ReadData(OpenDialog->FileName);
    Reload();
  }
}

//----------------------------- Save as:-------------------------------------

void __fastcall TMainForm::mnSaveAsClick(TObject *Sender)
{
  SaveDialog->Title = "Save file as...";
  SaveDialog->InitialDir = ExtractFilePath(ParamStr(0));
  SaveDialog->FileName = "Params1";
  SaveDialog->Filter = "Params files (*.par)|*.par|";
  SaveDialog->DefaultExt = "par";
  if(SaveDialog->Execute())
  {
    SaveData(SaveDialog->FileName);
  }
}

//------------------------------- Exit: -------------------------------------

void __fastcall TMainForm::mnExitClick(TObject *Sender)
{
  Close();
}

//---------------------------- Lock, Mute: ----------------------------------

void __fastcall TMainForm::mnLockClick(TObject *Sender)
{
  Lock = mnLock->Checked;
  Mute = mnMute->Checked;
  Cmd_SetMode(Lock);
}

//----------------------------- Rewiew: -------------------------------------

void __fastcall TMainForm::mnReviewClick(TObject *Sender)
{
  Recall();
  GroupBox1->Enabled = !mnReview->Checked;
  GroupBox2->Enabled = !mnReview->Checked;
  GroupBox3->Enabled = !mnReview->Checked;
  GroupBox4->Enabled = !mnReview->Checked;
}

//------------------------------ Recall: ------------------------------------

void __fastcall TMainForm::mnRecallClick(TObject *Sender)
{
  Recall();
}

//------------------------------ Reload: ------------------------------------

void __fastcall TMainForm::mnReloadClick(TObject *Sender)
{
  Reload();
}

//-------------------------------- RMS: -------------------------------------

void __fastcall TMainForm::mnRmsClick(TObject *Sender)
{
  lbLevelA->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  lbLevelB->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  lbLevelC->Caption = mnRms->Checked? "RMS:" : "Amplitude:";
  ShowAll();
}

//----------------------------- Settings: -----------------------------------

void __fastcall TMainForm::SettingsClick(TObject *Sender)
{
  TWinControl *wc;
  wc = MainForm->ActiveControl;
  MainForm->ActiveControl = StatusBar;
  if(!SettingsForm) SettingsForm = new TSettingsForm(Application);
  SettingsForm->ShowModal();
  if(Autorecall->Checked) Recall(); else Reload();
  MainForm->ActiveControl = wc;
}

//------------------------------ About: -------------------------------------

void __fastcall TMainForm::mnAboutClick(TObject *Sender)
{
  if(!AboutForm) AboutForm = new TAboutForm(Application);
  AboutForm->ShowModal();
}

//------------------------------- Edit: -------------------------------------

void __fastcall TMainForm::DoEdit(TObject *Sender, int &ch, int &par, double &val)
{
  par = ((TComponent *) Sender)->Tag;
  val = ((TFloatEdit *) Sender)->Value;
  ch = OUT_A;
  if(par > 19) { ch = OUT_C; par = par - 20; }
  if(par > 9) { ch = OUT_B; par = par - 10; }
  if(par == PAR_F) val = val * 1E3;
  if(par == PAR_P) val = val * 1E1;
  if(par == PAR_A)
  {
    val = val * 1E2;
    if(mnRms->Checked) val = val * RMS2AMP;
  }
  ValOuts[ch][par] = val;
}

//--------------------------- Edit enter: -----------------------------------

void __fastcall TMainForm::feWaEnter(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  int ch, par; double val;
  DoEdit(Sender, ch, par, val);
  char s = Mute? 0 : BEEP;
  Cmd_SetPar(ch, DRAW | s | par, Val2Par(val));
  ShowAll();
  FLock = 0;
}

//-------------------------- Edit changed: -----------------------------------

void __fastcall TMainForm::feWaChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  int ch, par; double val;
  DoEdit(Sender, ch, par, val);
  Cmd_SetPar(ch, DRAW | par, Val2Par(val));
  ShowAll();
  FLock = 0;
}

//------------------------------- Combo: ------------------------------------

void __fastcall TMainForm::DoCombo(TObject *Sender, int &ch, int &par, int &val)
{
  par = ((TComponent *) Sender)->Tag;
  val = ((TComboBox *) Sender)->ItemIndex;
  ch = OUT_A;
  if(par > 19) { ch = OUT_C; par = par - 20; }
  if(par > 9) { ch = OUT_B; par = par - 10; }
  ValOuts[ch][par] = val;
}

//---------------------------- Combo enter: ---------------------------------

void __fastcall TMainForm::cbMaEnter(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  int ch, par, val;
  DoCombo(Sender, ch, par, val);
  char s = Mute? 0 : BEEP;
  Cmd_SetPar(ch, DRAW | s | par, val);
  Cmd_GetPar(ch, PAR_A, ValOuts[ch][PAR_A]);
  ShowAll();
  FLock = 0;
}

//---------------------------- Combo change: --------------------------------

void __fastcall TMainForm::cbMaChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  int ch, par, val;
  DoCombo(Sender, ch, par, val);
  Cmd_SetPar(ch, DRAW | par, val);
  Cmd_GetPar(ch, PAR_A, ValOuts[ch][PAR_A]);
  ShowAll();
  FLock = 0;
}

//---------------------------- Save Preset: ---------------------------------

void __fastcall TMainForm::btSaveClick(TObject *Sender)
{
  TCursor Save_Cursor = Screen->Cursor;
  try
  {
    Screen->Cursor = crHourGlass;
    char s = Mute? 0 : BEEP;
    Cmd_SetPar(SETUP, s | SAVE_P, cbPre->ItemIndex);
    Sleep(500);
  }
    __finally { Screen->Cursor = Save_Cursor; }
}

//---------------------------- Read Preset: ---------------------------------

void __fastcall TMainForm::btReadClick(TObject *Sender)
{
  char s = Mute? 0 : BEEP;
  Cmd_SetPar(SETUP, s | READ_P, cbPre->ItemIndex);
  Recall();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
  int ch, par, val;
  Timer1->Enabled = 0;
  if(FConnected)
    try
    {
      if(mnReview->Checked)
      {
        Cmd_GetSelPar(ch, par, val);
        if((ch == OUT_A || ch == OUT_B || ch == OUT_C) &&
            par >= 0 && par < POUTS_MAX)
          ValOuts[ch][par] = val;
        ShowAll();
      }
      else Cmd_GetMode(val);
    }
    catch (const EHardwareException &e)
    {
      SG633_CloseDevice();
      FConnected = 0;
      StatusBar->SimpleText = "OFF-Line";
    }
  if(!FConnected)
  {
    FConnected = SG633_OpenDevice();
    if(FConnected)
    {
      try
      {
        Cmd_SetMode(Lock);
        if(Autorecall->Checked) Recall(); else Reload();
        StatusBar->SimpleText = "ON-Line";
      }
      catch (const EHardwareException &e)
      {
        SG633_CloseDevice();
        FConnected = 0;
        StatusBar->SimpleText = "OFF-Line";
      }
    }
  }
  Timer1->Enabled = 1;
}

//---------------------------------------------------------------------------

