#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "Settings.h"
#include "Commands.h"
#include "sg633usb.h"

#pragma package(smart_init)
#pragma link "FloatEdit"
#pragma resource "*.dfm"
TSettingsForm *SettingsForm;

//---------------------------------------------------------------------------

__fastcall TSettingsForm::TSettingsForm(TComponent* Owner)
  : TForm(Owner)
{
}

//----------------------------- Form create: --------------------------------

void __fastcall TSettingsForm::FormCreate(TObject *Sender)
{
  int val;
  FLock = 1;

  Cmd_GetPar(SETUP, LCD_C, val);
  sbContr->Position = val;
  feContr->Value = val;
  Cont = val;

  ReadCalib();

  FLock = 0;
  FModified = 0;
}

//----------------------------- Form close: ---------------------------------

void __fastcall TSettingsForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  if(FModified)
  {
    char s = MainForm->Mute? 0 : BEEP;
    Cmd_SetPar(CALIB, CAL_A, CalA);
    Cmd_SetPar(CALIB, CAL_B, CalB);
    Cmd_SetPar(CALIB, CAL_C, CalC);
    Cmd_SetPar(CALIB, DRAW | s | CAL_F, CalF);
  }
  Action = caFree;
  SettingsForm = NULL;
}

//-------------------------------- Ok: --------------------------------------

void __fastcall TSettingsForm::btOkClick(TObject *Sender)
{
  Cmd_SetPar(SETUP, SAVE_S, 0);
}

//----------------------------- Cancel: -------------------------------------

void __fastcall TSettingsForm::btCancelClick(TObject *Sender)
{
  Cmd_SetPar(SETUP, LCD_C, Cont);
  ModalResult = mrCancel;
}

//-------------------------------- ESC: -------------------------------------

void __fastcall TSettingsForm::FormKeyPress(TObject *Sender, char &Key)
{
  if (Key == VK_ESCAPE)
    btCancelClick(NULL);
}

//-------------------------- Scroll contrast: -------------------------------

void __fastcall TSettingsForm::sbContrChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  feContr->Value = sbContr->Position;
  Cmd_SetPar(SETUP, LCD_C, sbContr->Position);
  FLock = 0;
}

//---------------------------- Edit contrast: -------------------------------

void __fastcall TSettingsForm::feContrChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  sbContr->Position = feContr->Value + 0.5;
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(SETUP, s | LCD_C, sbContr->Position);
  FLock = 0;
}

//----------------------------- Read Calib: ---------------------------------

void __fastcall TSettingsForm::ReadCalib(void)
{
  int val;
  Cmd_GetPar(CALIB, CAL_F, val);
  sbCf->Position = val;
  feCf->Value = val / 10.0;
  CalF = val;

  Cmd_GetPar(CALIB, CAL_A, val);
  sbCa->Position = val;
  feCa->Value = val / 100.0;
  CalA = val;

  Cmd_GetPar(CALIB, CAL_B, val);
  sbCb->Position = val;
  feCb->Value = val / 100.0;
  CalB = val;

  Cmd_GetPar(CALIB, CAL_C, val);
  sbCc->Position = val;
  feCc->Value = val / 100.0;
  CalC = val;
}

//--------------------------- Refresh Calib: --------------------------------

void __fastcall TSettingsForm::btRefreshClick(TObject *Sender)
{
  ReadCalib();
}

//---------------------------- Enter Calib_F: -------------------------------

void __fastcall TSettingsForm::feCfEnter(TObject *Sender)
{
  if(FLock) return;
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(CALIB, DRAW | s | CAL_F, sbCf->Position);
}

//--------------------------- Scroll Calib_F: -------------------------------

void __fastcall TSettingsForm::sbCfChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  feCf->Value = sbCf->Position / 10.0;
  Cmd_SetPar(CALIB, DRAW | CAL_F, sbCf->Position);
  FModified = 1;
  FLock = 0;
}

//---------------------------- Edit Calib_F: --------------------------------

void __fastcall TSettingsForm::feCfChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  sbCf->Position = feCf->Value * 10.0 + ((feCf->Value > 0)? 0.5 : -0.5);
  if(sbCf->Position != CalF)
  {
    Cmd_SetPar(CALIB, DRAW | CAL_F, sbCf->Position);
    FModified = 1;
  }
  FLock = 0;
}

//---------------------------- Enter Calib_A: -------------------------------

void __fastcall TSettingsForm::feCaEnter(TObject *Sender)
{
  if(FLock) return;
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(CALIB, DRAW | s | CAL_A, sbCa->Position);
}

//--------------------------- Scroll Calib_A: -------------------------------

void __fastcall TSettingsForm::sbCaChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  feCa->Value = sbCa->Position / 100.0;
  Cmd_SetPar(CALIB, DRAW | CAL_A, sbCa->Position);
  FModified = 1;
  FLock = 0;
}

//---------------------------- Edit Calib_A: --------------------------------

void __fastcall TSettingsForm::feCaChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  sbCa->Position = feCa->Value * 100.0 + ((feCa->Value > 0)? 0.5 : -0.5);
  if(sbCa->Position != CalA)
  {
    Cmd_SetPar(CALIB, DRAW | CAL_A, sbCa->Position);
    FModified = 1;
  }
  FLock = 0;
}

//---------------------------- Enter Calib_B: -------------------------------

void __fastcall TSettingsForm::feCbEnter(TObject *Sender)
{
  if(FLock) return;
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(CALIB, DRAW | s | CAL_B, sbCb->Position);
}

//--------------------------- Scroll Calib_B: -------------------------------

void __fastcall TSettingsForm::sbCbChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  feCb->Value = sbCb->Position / 100.0;
  Cmd_SetPar(CALIB, DRAW | CAL_B, sbCb->Position);
  FModified = 1;
  FLock = 0;
}

//---------------------------- Edit Calib_B: --------------------------------

void __fastcall TSettingsForm::feCbChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  sbCb->Position = feCb->Value * 100.0 + ((feCb->Value > 0)? 0.5 : -0.5);
  if(sbCb->Position != CalB)
  {
    Cmd_SetPar(CALIB, DRAW | CAL_B, sbCb->Position);
    FModified = 1;
  }
  FLock = 0;
}

//---------------------------- Enter Calib_C: -------------------------------

void __fastcall TSettingsForm::feCcEnter(TObject *Sender)
{
  if(FLock) return;
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(CALIB, DRAW | s | CAL_C, sbCc->Position);
}

//--------------------------- Scroll Calib_C: -------------------------------

void __fastcall TSettingsForm::sbCcChange(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  feCc->Value = sbCc->Position / 100.0;
  Cmd_SetPar(CALIB, DRAW | CAL_C, sbCc->Position);
  FModified = 1;
  FLock = 0;
}

//---------------------------- Edit Calib_C: --------------------------------

void __fastcall TSettingsForm::feCcChanged(TObject *Sender)
{
  if(FLock) return;
  FLock = 1;
  sbCc->Position = feCc->Value * 100.0 + ((feCc->Value > 0)? 0.5 : -0.5);
  if(sbCc->Position != CalC)
  {
    Cmd_SetPar(CALIB, DRAW | CAL_C, sbCc->Position);
    FModified = 1;
  }
  FLock = 0;
}

//----------------------------- Save Calib: ---------------------------------

void __fastcall TSettingsForm::btSaveClick(TObject *Sender)
{
  char s = MainForm->Mute? 0 : BEEP;
  Cmd_SetPar(CALIB, s | CAL_S, 0);
  FModified = 0;
}

//---------------------------------------------------------------------------

